
// Exp4jTest.java
// Andrew Davison, ad@coe.psu.ac.th, June 2023

/* A set of tests for Expr4j 
   (https://gitlab.com/riddler_arg/exp4j), a library for 
   parsing and evaluating mathematical expression strings.

   Usage:
     > compile Exp4jTest.java
     > run Exp4jTest
*/

import net.objecthunter.exp4j.*;
import net.objecthunter.exp4j.function.*;
import net.objecthunter.exp4j.operator.*;
import net.objecthunter.exp4j.extras.*;


public class Exp4jTest 
{

  public static void main(String[] args) 
  {
    Expression expr = new ExpressionBuilder("gcd(13,39)")
               .function(FunctionsMisc.GCD)
               .build();
    double result = expr.evaluate();
    System.out.println(result);   // prints 13

    expr = new ExpressionBuilder("5 != 7")
               .operators(OperatorsComparison.getOperators())
               .build();
    result = expr.evaluate();
    System.out.println(result);   // 1

    expr = new ExpressionBuilder("3x+2y")
      .variables("x", "y")
      .build()
      .setVariable("x", 2)
      .setVariable("y", 3);
    result = expr.evaluate();
    System.out.println(result);   // 12

    expr = new ExpressionBuilder(
            "sin(x)*sin(x)+cos(x)*cos(x)")
      .variables("x")
      .build()
      .setVariable("x", 0.5);
    result = expr.evaluate();
    System.out.println(result);  // 1

    expr = new ExpressionBuilder("log10(1000)")
      .functions(Expr4jExtras.funcs)
      .build();
    result = expr.evaluate();
    System.out.println(result);  // 3

    expr = new ExpressionBuilder("8 >> 2")
            .operators(Expr4jExtras.ops)
            .build();
    result = expr.evaluate();
    System.out.println(result);  // 2

    expr = new ExpressionBuilder("!(2 && 3)")
            .operators(Expr4jExtras.ops)
            .build();
    result = expr.evaluate();
    System.out.println(result);  //0
  }

} // end of Exp4jTest class