
// Expr4jExtras.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, Nov. 2022

/* Additional exp4j functions and operators
   (https://gitlab.com/riddler_arg/exp4j)

   Usage:
     > compile.bat Expr4jExtras
*/

import java.io.*;
import java.util.*;

import net.objecthunter.exp4j.function.*;
import net.objecthunter.exp4j.operator.*;



public class Expr4jExtras
{

  public static Function atan2 = new Function("atan2", 2) {
    public double apply(double[] args) 
    {  return Math.atan2(args[0], args[1]); }
  };

  public static Function hypot = new Function("hypot", 2) {
    public double apply(double[] args) 
    {  return Math.hypot(args[0], args[1]); }
  };

  public static Function random = new Function("random", 0) {
    public double apply(double[] args) 
    {  return (2*Math.random())-1; }   // between -1.0 - 1.0
  };


  public static Function[] funcs = new Function[] { 
                           atan2, hypot, random };


  // ------

  public static Operator spow = new Operator("**", 2, true, 
                             Operator.PRECEDENCE_ADDITION-1) {
    // star power
    public double apply(double[] vals) 
    {  return  Math.pow(vals[0], vals[1]);  }
  };


  public static Operator logNot = new Operator("!", 1, false, 
                             Operator.PRECEDENCE_ADDITION-1) {
    public double apply(double[] vals) 
    {  
      boolean b0 = (vals[0] == 0) ? false : true;
      boolean res = !b0;
      return ((res) ? 1.0 : 0.0);
    }
  };

  public static Operator logAnd = new Operator("&&", 2, true, 
                             Operator.PRECEDENCE_ADDITION-1) {
    public double apply(double[] vals) 
    {  
      boolean b0 = (vals[0] == 0) ? false : true;
      boolean b1 = (vals[1] == 0) ? false : true;
      boolean res = b0 && b1;
      return ((res) ? 1.0 : 0.0);
    }
  };

  public static Operator logOr = new Operator("||", 2, true, 
                             Operator.PRECEDENCE_ADDITION-1) {
    public double apply(double[] vals) 
    {  
      boolean b0 = (vals[0] == 0) ? false : true;
      boolean b1 = (vals[1] == 0) ? false : true;
      boolean res = b0 || b1;
      return ((res) ? 1.0 : 0.0);
    }
  };


  public static Operator lShf = new Operator("<<", 2, true, 
                             Operator.PRECEDENCE_ADDITION-1) {
    // left shift
    public double apply(double[] vals) 
    {  return  (((int)Math.round(vals[0])) << 
                ((int)Math.round(vals[1])));  }
  };

  public static Operator srShf = new Operator(">>", 2, true, 
                             Operator.PRECEDENCE_ADDITION-1) {
    // signed right shift
    public double apply(double[] vals) 
    {  return  (((int)Math.round(vals[0])) >> 
                ((int)Math.round(vals[1])));  }
  };

  public static Operator urShf = new Operator(">>>", 2, true, 
                             Operator.PRECEDENCE_ADDITION-1) {
    // unsigned right shift
    public double apply(double[] vals) 
    {  return  (((int)Math.round(vals[0])) >>> 
                ((int)Math.round(vals[1])));  }
  };

  public static Operator[] ops = new Operator[] {
           spow, 
           logNot, logAnd, logOr, 
           lShf, srShf, urShf };


}  // end of Expr4jExtras class